from django.db import models
from django.contrib.auth.models import User
from django.utils.text import slugify
from django.db.models.signals import pre_save
from django.dispatch import receiver
from PIL import Image


class Customer(models.Model):
    user = models.OneToOneField(User, null=True, blank=True, on_delete=models.CASCADE)
    name = models.CharField(max_length=200, null=True)
    email = models.EmailField()

    def __str__(self):
        return self.name or ''
    

class Classification(models.Model):
    name = models.CharField(max_length=50)
    description = models.TextField(null=True, blank=True)
    slug = models.SlugField(max_length=200, unique=True, null=True, blank=True)

    class Meta:
        verbose_name_plural = "Classifications"

    def __str__(self):
        return self.name
    
    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.name)
        super().save(*args, **kwargs)



class ProductCategory(models.Model):    
    name = models.CharField(max_length=50)
    image = models.ImageField(upload_to='products', null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    slug = models.SlugField(max_length=200, unique=True, null=True, blank=True)
    classifications = models.ManyToManyField(Classification, related_name='categories')

    def __str__(self):
        return self.name
    
    class Meta:
        verbose_name_plural = "Categories"

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.name)
        super().save(*args, **kwargs)


class Product(models.Model):
    one = "1"
    two = "2"
    three = "3"
    four = "4"
    five = "5"
    Rating = [
        (one, 1),
        (two, 2),
        (three, 3),
        (four, 4),
        (five, 5),
    ]
    category = models.ForeignKey(ProductCategory, default="", on_delete=models.CASCADE, blank=True, null=True)
    classification = models.ForeignKey(Classification, default="", on_delete=models.CASCADE, blank=True, null=True)
    name = models.CharField(max_length=200)
    price = models.DecimalField(max_digits=5, decimal_places=2)
    digital = models.BooleanField(default=False, null=True, blank=True)
    image = models.ImageField(upload_to='products', null=True, blank=True)
    product_slug = models.SlugField(max_length=50, blank=True, null=True)
    product_rating = models.CharField(max_length=1, choices=Rating, default="5", blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)
        if self.image:
            img = Image.open(self.image.path)
            target_size = (800, 800)  # Resize to 800x800 pixels for main product images
            img = img.resize(target_size, Image.ANTIALIAS)
            img.save(self.image.path)

    def __str__(self):
        return self.name

    @property
    def imageURL(self):
        try:
            url = self.image.url
        except:
            url = ''
        return url

@receiver(pre_save, sender=Product)
def prepopulate_slug(sender, instance, **kwargs):
    if not instance.product_slug:
        instance.product_slug = slugify(instance.name+(str(instance.id)))


class productDetails(models.Model):
    name = models.ForeignKey(Product, default="", on_delete=models.CASCADE)
    brand = models.CharField(max_length=50)
    size = models.CharField(max_length=30)
    color = models.CharField(max_length=100, blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    features = models.TextField(blank=True, null=True)
    specifications = models.TextField(blank=True, null=True)
    shippingOptions = models.TextField(blank=True, null=True)    

    class Meta:
        verbose_name_plural = "Product Details"


class ProductImage(models.Model):
    product = models.ForeignKey(Product, related_name='images', on_delete=models.CASCADE)
    image = models.ImageField(upload_to='products', null=True, blank=True)

    def __str__(self):
        return f"{self.product.name} Image"

    @property
    def imageURL(self):
        try:
            url = self.image.url
        except:
            url = ''
        return url


class Order(models.Model):
    customer = models.ForeignKey(Customer, on_delete=models.SET_NULL, null=True, blank=True)
    date_ordered = models.DateTimeField(auto_now_add=True)
    complete = models.BooleanField(default=False)
    transaction_id = models.CharField(max_length=100, null=True)

    def __str__(self):
        return str(self.id) or ''

    @property
    def shipping(self):
    	shipping = False
    	orderitems = self.orderitem_set.all()
    	for i in orderitems:
    		if i.product.digital == False:
    			shipping = True
    	return shipping

    @property
    def get_cart_total(self):
        orderitems = self.orderitem_set.all()
        total = sum([item.get_total for item in orderitems])
        return total

    @property
    def get_cart_items(self):
        orderitems = self.orderitem_set.all()
        total = sum([item.quantity for item in orderitems])
        return total


class OrderItem(models.Model):
    product = models.ForeignKey(Product, on_delete=models.SET_NULL, null=True)
    order = models.ForeignKey(Order, on_delete=models.SET_NULL, null=True)
    quantity = models.IntegerField(default=0, null=True, blank=True)
    date_added = models.DateTimeField(auto_now_add=True)

    @property
    def get_total(self):
        total = self.product.price * self.quantity
        return total


class ShippingAddress(models.Model):
    customer = models.ForeignKey(Customer, on_delete=models.SET_NULL, null=True)
    order = models.ForeignKey(Order, on_delete=models.SET_NULL, null=True)
    address = models.CharField(max_length=200, null=False)
    country = models.CharField(max_length=200, null=True)
    city = models.CharField(max_length=200, null=False)
    state = models.CharField(max_length=200, null=False)
    zipcode = models.CharField(max_length=200, null=True)
    info = models.CharField(max_length=200, null=True)
    date_added = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.address


class Slider(models.Model):
    name = models.CharField(max_length=50, blank=True, null=True)
    image = models.ImageField(upload_to='slider' ,null=True, blank=True)
   
    def __str__(self):
        return self.name

    @property
    def imageURL(self):
        try:
            url = self.image.url
        except:
            url = ''
        return url

    @property
    def product_url(self):
        prod_url = self.name + str(self.id)
        return str(prod_url)

