from django.test import TestCase, Client
from django.urls import reverse
from django.contrib.auth.models import User
from .models import Customer, Product, ProductCategory, Classification, Order, OrderItem, ShippingAddress, Slider
from .views import Home, product_list, product_detail, Cart, Checkout, updateItem, processOrder
from django.contrib.admin.sites import AdminSite
from .admin import OrderAdmin, ProductAdmin, CustomerAdmin


class ModelTests(TestCase):

    def setUp(self):
        self.user = User.objects.create_user(username='testuser', password='12345')
        self.customer = Customer.objects.create(user=self.user, name='Test Customer', email='test@example.com')
        self.category = ProductCategory.objects.create(name='Test Category')
        self.classification = Classification.objects.create(name='Test Classification')
        self.product = Product.objects.create(name='Test Product', price=10.00, category=self.category, classification=self.classification)
        self.order = Order.objects.create(customer=self.customer, complete=False)
        self.order_item = OrderItem.objects.create(product=self.product, order=self.order, quantity=1)
        self.shipping_address = ShippingAddress.objects.create(customer=self.customer, order=self.order, address='123 Test St', city='Test City', state='Test State', country='Test Country', zipcode='12345')

    def test_customer_creation(self):
        self.assertEqual(self.customer.name, 'Test Customer')
        self.assertEqual(self.customer.email, 'test@example.com')

    def test_product_creation(self):
        self.assertEqual(self.product.name, 'Test Product')
        self.assertEqual(self.product.price, 10.00)

    def test_order_creation(self):
        self.assertEqual(self.order.customer, self.customer)
        self.assertFalse(self.order.complete)

    def test_order_item_creation(self):
        self.assertEqual(self.order_item.product, self.product)
        self.assertEqual(self.order_item.quantity, 1)

    def test_shipping_address_creation(self):
        self.assertEqual(self.shipping_address.address, '123 Test St')
        self.assertEqual(self.shipping_address.city, 'Test City')


class ViewTests(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_user(username='testuser', password='12345')
        self.customer = Customer.objects.create(user=self.user, name='Test Customer', email='test@example.com')
        self.category = ProductCategory.objects.create(name='Test Category')
        self.classification = Classification.objects.create(name='Test Classification')
        self.product = Product.objects.create(name='Test Product', price=10.00, category=self.category, classification=self.classification)
        self.order = Order.objects.create(customer=self.customer, complete=False)
        self.order_item = OrderItem.objects.create(product=self.product, order=self.order, quantity=1)
        self.shipping_address = ShippingAddress.objects.create(customer=self.customer, order=self.order, address='123 Test St', city='Test City', state='Test State', country='Test Country', zipcode='12345')

    def test_home_view(self):
        response = self.client.get(reverse('home'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'store/main.html')

    def test_product_list_view(self):
        response = self.client.get(reverse('product_list'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'store/product_list.html')

    def test_product_detail_view(self):
        response = self.client.get(reverse('product_detail', args=[self.product.product_slug]))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'store/productdetails.html')

    def test_cart_view(self):
        response = self.client.get(reverse('cart'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'store/cart.html')

    def test_checkout_view(self):
        response = self.client.get(reverse('checkout'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'store/checkout.html')

class URLTests(TestCase):

    def test_home_url(self):
        response = self.client.get('/')
        self.assertEqual(response.status_code, 200)

    def test_product_list_url(self):
        response = self.client.get('/products/')
        self.assertEqual(response.status_code, 200)

    def test_product_detail_url(self):
        response = self.client.get(f'/products/{self.product.product_slug}/')
        self.assertEqual(response.status_code, 200)

    def test_cart_url(self):
        response = self.client.get('/cart/')
        self.assertEqual(response.status_code, 200)

    def test_checkout_url(self):
        response = self.client.get('/checkout/')
        self.assertEqual(response.status_code, 200)


class AdminTests(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(username='admin', password='admin')
        self.client.login(username='admin', password='admin')
        self.customer = Customer.objects.create(user=self.user, name='Test Customer', email='test@example.com')
        self.order = Order.objects.create(customer=self.customer, complete=False)

    def test_order_admin(self):
        order_admin = OrderAdmin(Order, AdminSite())
        self.assertEqual(order_admin.get_customer(self.order), 'admin')
        self.assertEqual(order_admin.get_customer_email(self.order), 'test@example.com')

class TemplateTests(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_user(username='testuser', password='12345')
        self.customer, created = Customer.objects.get_or_create(user=self.user, defaults={'name': 'Test Customer', 'email': 'test@example.com'})
        self.category = ProductCategory.objects.create(name='Test Category')
        self.classification = Classification.objects.create(name='Test Classification')
        self.product = Product.objects.create(name='Test Product', price=10.00, category=self.category, classification=self.classification)
        self.order = Order.objects.create(customer=self.customer, complete=False)
        self.order_item = OrderItem.objects.create(product=self.product, order=self.order, quantity=1)
        self.shipping_address = ShippingAddress.objects.create(customer=self.customer, order=self.order, address='123 Test St', city='Test City', state='Test State', country='Test Country', zipcode='12345')

    def test_home_template(self):
        response = self.client.get(reverse('home'))
        self.assertContains(response, 'Test Product')

    def test_product_list_template(self):
        response = self.client.get(reverse('product_list'))
        self.assertContains(response, 'Test Product')

    def test_product_detail_template(self):
        response = self.client.get(reverse('product_detail', args=[self.product.product_slug]))
        self.assertContains(response, 'Test Product')

